#!/usr/bin/env python2

"""
Create torrent.
"""

import sys
import argparse
import better_bencode as bb
import abt.torrent as torrent
import abt.cli as cli

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('path', action='store', help="the path of file or directory to share")
    parser.add_argument('--add-tracker', action='append', dest='announce_list', metavar='TRACKER',
                                         help="add tracker (NOTE torrent without tracker can only be inferred from DHT)")
    parser.add_argument('--comment', action='store', help="add torrent file comment")
    parser.add_argument('--name', action='store', help="set name of shared file/directory")
    parser.add_argument('--private', action='store_true', help="flag for restricted access")
    parser.add_argument('--output', action='store', help="save torrent to file")
    args = parser.parse_args()

    torrent_data = torrent.create_torrent_data(args.path, name=args.name, announce_list=args.announce_list, private_flag=args.private)

    if args.comment:
        torrent_data['comment'] = args.comment

    out = sys.stdout

    if args.output:
        out = open(args.output, 'wb')

    out.write(bb.dumps(torrent_data))



#!/usr/bin/env python2

"""
Show aria2 status
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client
import abt.format as fmt

keys = ['gid', 'status', 'totalLength', 'completedLength', 'connections', 'numSeeders', 'seeder', 'dir', 'uploadSpeed', 'downloadSpeed']

def print_status(aria2, active=True, waiting=False, stopped=False):
    summary = aria2.getGlobalStat()

    print "aria2 ({numActive} active, {numWaiting} waiting, {numStopped} stopped)".format(**summary)

    def print_tasks(pattern, tasks):
        if len(tasks) == 0:
            print "  (empty)"
        for task in tasks:
            print pattern % fmt.Task(task)

    if active:
        print
        print "Active Tasks (gid/sz/prg/conn/up/down):\n"
        print_tasks("  %(gid)s %(sz)s %(prg)s  %(conn)s %(up)s %(down)s", aria2.tellActive(keys))

    if waiting:
        print
        print "Waiting Tasks (gid/sz/prg/dir):\n"
        print_tasks("  %(gid)s %(sz)s %(prg)s  %(dir)s", aria2.tellWaiting(0, 999, keys))

    if stopped:
        print
        print "Stopped Tasks (gid/sz/prg/dir):\n"
        print_tasks("  %(gid)s %(sz)s %(prg)s  %(dir)s", aria2.tellStopped(0, 999, keys))

    print
    print u"\u21f1 {}".format(fmt.format_speed(summary['uploadSpeed']))
    print u"\u21f2 {}".format(fmt.format_speed(summary['downloadSpeed']))
    print

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('-a', '--all', action='store_true', help="show all tasks")
    parser.add_argument('--waiting', action='store_true', help="show waiting tasks")
    parser.add_argument('--stopped', action='store_true', help="show stopped tasks")
    parser.add_argument('--no-active', action='store_false', dest='active', help="hide active tasks")
    conn, extra = cli.parse_connection_options()
    args, extra = parser.parse_known_args(extra)
    aria2, _ = client.connect(**conn)

    try:
        version_info = aria2.getVersion()
    except:
        cli.error("aria2 is not running.")

    print_status(aria2, active=args.all or args.active,
                       waiting=args.all or args.waiting,
                       stopped=args.all or args.stopped)




from xmlrpclib import SafeTransport as XMLSafeTransport
from jsonrpclib.jsonrpc import TransportMixIn

class SafeTransport(TransportMixIn, XMLSafeTransport):
    def __init__(self, context=None):
        TransportMixIn.__init__(self)
        XMLSafeTransport.__init__(self, context)




#!/usr/bin/env python2

import os
import hashlib

def get_pieces(files, piece_length=0x8000):
    if isinstance(files, basestring):
        files = [ files ]
    piece_data = ""
    for f in files:
        current_file = open(f, 'rb')
        while True:
            piece_data += current_file.read(piece_length - len(piece_data))
            if len(piece_data) != piece_length:
                current_file.close()
                break
            yield piece_data
            piece_data = ""
    if piece_data:
        yield piece_data

def get_files(path):
    for (dirname, dirs, files) in os.walk(path):
        for f in files:
            yield os.path.join(dirname, f)

def get_checksum(data):
    m = hashlib.sha1()
    m.update(data)
    return m.digest()

def create_torrent_data(path, name=None, announce=None, announce_list=None, private_flag=False, piece_length=0x8000):
    info = {}
    info['name'] = name if name else os.path.basename(path)
    info['piece length'] = piece_length
    if private_flag:
        info['private'] = 1
    if os.path.isfile(path):
        info['length'] = os.path.getsize(path);
        pieces_data = get_pieces(path)
    else:
        files = [ f for f in get_files(path) ]
        info['files'] = [ { 'path': [ os.path.relpath(f, path) ], 'length': os.path.getsize(f) } for f in files ]
        pieces_data = get_pieces(files)
    info['pieces'] = ''.join(map(get_checksum, pieces_data))
    torrent = {}
    if announce:
        torrent['announce'] = announce
    if announce_list:
        torrent['announce-list'] = [ [announce] for announce in announce_list ]
    torrent['info'] = info
    return torrent



#!/usr/bin/env python2

"""
This is a template file of abt command
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('arg1', action='store', help="balabalah")
    parser.add_argument('arg2', action='store', help="balabalah")
    parser.add_argument('--opt1', action='store', help="balabalah")
    parser.add_argument('--opt2', action='store', help="balabalah")
    args = parser.parse_args()




from humanize import naturalsize

column_defs = {}

class ColumnDef:

    def __init__(self, fmt, keys, transform=None):
        self.fmt = fmt
        self.keys = keys if isinstance(keys, list) else [keys]
        self.transform = transform

    def __call__(self, data):
        values = map(lambda k: data[k], self.keys)
        if self.transform:
            values = [ self.transform(*values) ]
        return self.fmt.format(values[0])

def add_column_def(name, fmt="{}", keys=None, transform=None):
    column_defs[name] = ColumnDef(fmt, keys if keys else name, transform)

def nozero(func):
    def wrapper(*args):
        if int(args[0]):
            return func(*args)
        return "-"
    return wrapper

def format_status(status):
    pass

def format_speed(value):
    return "%s/s" % naturalsize(value)

def format_progress(total, prog):
    return "{:.0f}%".format(round(100 * float(prog) / float(total)))

def format_conn(connections, numSeeders, seeder):
    if seeder == 'true':
        return connections
    else:
        return "%s(%s)" % (connections, numSeeders)

add_column_def('gid', '{:<16}')
add_column_def('status', transform=format_status)
add_column_def('sz', '{:>10}', 'totalLength', transform=nozero(naturalsize))
add_column_def('prg', '{:>5}', ['totalLength','completedLength'], transform=nozero(format_progress))
add_column_def('conn', '{:<6}', ['connections','numSeeders', 'seeder'], transform=format_conn)
add_column_def('up', '{:<11}', 'uploadSpeed', transform=nozero(format_speed))
add_column_def('down', '{:<11}', 'downloadSpeed', transform=nozero(format_speed))
add_column_def('dir', u'{}')

class Task(dict):

    def __init__(self, data):
        self.data = data

    def __contains__(self, key):
        return key in column_defs or super.__contains__(self, key)

    def __getitem__(self, key):
        if key in column_defs:
            return column_defs[key](self.data)
        return super.__getitem__(self, key)



#!/usr/bin/env python2

"""
Remove download task
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('gid', action='store', help="gid of the task to remove")
    parser.add_argument('-f', '--force', action='store_true', help="remove task immediately without sending stats to tracker")
    conn, extra = cli.parse_connection_options()
    args, extra = parser.parse_known_args(extra)

    aria2, _ = client.connect(**conn)

    status = aria2.tellStatus(args.gid, ['status'])['status']

    if status in ['active', 'waiting', 'paused']:
        if args.force:
            aria2.forceRemove(args.gid)
        else:
            aria2.remove(args.gid)

    print aria2.removeDownloadResult(args.gid)



import ssl
import jsonrpclib
import transport

def connect(url="http://localhost:6800/jsonrpc", ca_file=None, no_verify=False):
    if ca_file or no_verify:
        if no_verify:
            ssl_context = ssl._create_unverified_context()
        elif ca_file:
            ssl_context = ssl.create_default_context()
            ssl_context.load_verify_locations(cafile=ca_file)
        transp= transport.SafeTransport(context=ssl_context)
        server = jsonrpclib.Server(url, transport=transp)
    else:
        server = jsonrpclib.Server(url)
    return (server.aria2, server)




#!/usr/bin/env python2

"""
Add BitTorrent download task
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client
import base64
import os
import tempfile

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('torrent', nargs='?', action='store', help="path to torrent file")
    parser.add_argument('--uri', action='store', help="load torrent file from uri")
    conn, extra = cli.parse_connection_options()
    options, extra = cli.parse_option_dict(extra)
    args = parser.parse_args(extra)

    if (not args.torrent) == (not args.uri):
        parser.error("Exactly one of torrent file path or --uri is expected.")

    aria2, _ = client.connect(**conn)

    if args.uri:
        _, torrent_file = tempfile.mkstemp()
        fmt = "aria2c --no-conf --allow-overwrite=true --follow-torrent=false --async-dns=false --dir=%s --out=%s \"%s\""
        cmd = fmt % (os.path.dirname(torrent_file), os.path.basename(torrent_file), args.uri)
        ret = os.system(cmd)
        if ret != 0:
            cli.error("Failed. Cannot fetch torrent from %s" % args.uri)
    else:
        torrent_file = args.torrent

    torrent_data = open(torrent_file, 'rb').read()
    torrent_b64 = base64.b64encode(torrent_data)
    print aria2.addTorrent(torrent_b64, [], options)




#!/usr/bin/env python2

"""
Get/set aria2 options
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client
import base64
import os
import tempfile

def print_options(options):
    for k, v in options.items():
        print "--%s=%s" % (k, v)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('gid', nargs='?', action='store', help="gid to show/change options of")
    conn, extra = cli.parse_connection_options()
    options, extra = cli.parse_option_dict(extra)
    args = parser.parse_args(extra)
    aria2, _ = client.connect(**conn)

    if not args.gid:
        if options:
            print aria2.changeGlobalOption(options)
        else:
            print_options(aria2.getGlobalOption())
    else:
        if options:
            print aria2.changeOption(args.gid, options)
        else:
            print_options(aria2.getOption(args.gid))




#!/usr/bin/env python

from setuptools import setup

setup(name="abt",
      version="0.1.1",
      description="BitTorrent workflow with aria2.",
      author="Kamikat",
      author_email="kamikat@banana.moe",
      url="https://github.com/kamikat/abt",
      install_requires=['jsonrpclib', 'better-bencode', 'humanize'],
      packages=["abt","abt.cli"],
      entry_points={
          'console_scripts': [ 'abt = abt:main' ] },
      license="MIT")



#!/usr/bin/env python2

"""
Put active task to waiting
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    parser.add_argument('gid', action='store', help="gid of the task to pause")
    conn, extra = cli.parse_connection_options()
    args, extra = parser.parse_known_args(extra)

    aria2, _ = client.connect(**conn)

    print aria2.pause(args.gid)




#!/usr/bin/env python2

"""
Show aria2 status in top-like style
"""

import argparse
import abt.cli as cli
import abt.rpc_client as client
import abt.cli.status
import sys
import time
import curses
import locale

class StdOutWrapper:
    text = ""
    def write(self,txt):
        self.text += txt
        self.text = '\n'.join(self.text.split('\n')[-30:])

def top(screen):
    screen.nodelay(True)
    try:
        while True:
            if screen.getch() == ord('q'):
                break
            screen.clear()
            scrout = StdOutWrapper()
            sys.stdout = scrout # override stdout
            abt.cli.status.print_status(aria2)
            screen.addstr(0, 0, scrout.text.encode('utf-8'))
            screen.refresh()
            time.sleep(1)
    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=cli.progname, description=__doc__.strip())
    conn, extra = cli.parse_connection_options()
    args, extra = parser.parse_known_args(extra)
    aria2, _ = client.connect(**conn)

    try:
        version_info = aria2.getVersion()
    except:
        cli.error("aria2 is not running.")

    locale.setlocale(locale.LC_ALL, '')
    curses.wrapper(top)




import sys
import subprocess
import argparse
import re
import abt.rpc_client as client

from os import path, listdir
from imp import load_source
from sys import exit as error

progname = path.splitext(path.basename(sys.argv[0]))[0]
progname = "abt %s" % progname if progname != "abt" else "abt"
cli_path = path.dirname(__file__)

def main():
    if len(sys.argv) > 1:
        command = path.join(cli_path, "%s.py" % sys.argv[1])
        if path.exists(command):
            try:
                ret = subprocess.call([sys.executable, command] + sys.argv[2:])
            except KeyboardInterrupt:
                ret = 130
            sys.exit(ret)
    print "usage: %s <command> [<args>]" % progname
    print "       %s <command> [--jsonrpc JSONRPC] [--no-verify] [--ca-certificate CA_CERTIFICATE] [<args>]" % progname
    print "       %s <command> [<args>] [--<key>=<value> [--<key>=<value> ...]]" % progname
    print "       %s <command> [-h]\n" % progname
    print "abt is a BitTorrent management workflow.\n"
    print "List of Commands:"
    for command_source in sorted(listdir(cli_path)):
        if command_source.endswith('.pyc') or command_source.startswith('__'):
            continue
        command = command_source.split('.')[0]
        command_file = path.join(cli_path, command_source)
        command_module = load_source(command, command_file)
        description = command_module.__doc__.strip().replace('\n', '\n' + ' ' * 20) if command_module.__doc__ else ""
        print "   %s %s" % ("{0: <16}".format(command), description)
    print

def parse_connection_options(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(prog="abt", usage="%(prog)s <command> [--jsonrpc JSONRPC] [--no-verify] [--ca-certificate CA_CERTIFICATE]", add_help=False)
    parser.add_argument("--jsonrpc", action='store', dest='url', default="http://localhost:6800/jsonrpc")
    parser.add_argument("--no-verify", action='store_true')
    parser.add_argument("--ca-certificate", action='store', dest='ca_file')
    args, extra = parser.parse_known_args(args)
    return vars(args), extra

class StoreToDict(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        d = getattr(namespace, self.dest)
        for opt in values:
            try:
                k,v = opt.split("=", 1)
            except:
                parser.error("invalid option '%s'" % opt)
            k = k.lstrip("-")
            d[k] = v
        setattr(namespace, self.dest, d)

def parse_option_dict(args):
    options = {}
    extra = []
    for arg in args:
        if re.match('--[a-zA-Z][a-zA-Z0-9-]*=.*', arg):
            k, v = arg.split("=", 1)
            options[k.lstrip('-')] = v
        else:
            extra.append(arg)
    return options, extra

